@@ -12,6 +12,8 @@ extern "C" {
 #include <afs/stds.h>
 }
 
+#include <windows.h>
+#include <windowsx.h>
 #include <WINNT/talocale.h>
 #include <stdlib.h>
 
@@ -181,7 +183,7 @@ void cdecl vFormatMultiString (LPTSTR *ppszTarget, BOOL fAddHead, LONG ids, LPCT
 void cdecl FormatMultiString (LPTSTR *ppszTarget, BOOL fAddHead, LPCTSTR pszTemplate, LPCTSTR pszFormat, ...)
 {
    va_list arg;
-   if (pszFormat != NULL)
+   //if (pszFormat != NULL)
       va_start (arg, pszFormat);
    vFormatMultiString (ppszTarget, fAddHead, (LONG)pszTemplate, pszFormat, arg);
 }
@@ -189,7 +191,7 @@ void cdecl FormatMultiString (LPTSTR *ppszTarget, BOOL fAddHead, LPCTSTR pszTemp
 void cdecl FormatMultiString (LPTSTR *ppszTarget, BOOL fAddHead, int idsTemplate, LPCTSTR pszFormat, ...)
 {
    va_list arg;
-   if (pszFormat != NULL)
+   //if (pszFormat != NULL)
       va_start (arg, pszFormat);
    vFormatMultiString (ppszTarget, fAddHead, (LONG)idsTemplate, pszFormat, arg);
 }
@@ -212,7 +214,7 @@ void cdecl vFormatMultiString (LPTSTR *ppszTarget, BOOL fAddHead, int idsTemplat
 LPTSTR cdecl FormatString (LPCTSTR psz, LPCTSTR pszFmt, ...)
 {
    va_list arg;
-   if (pszFmt != NULL)
+   //if (pszFmt != NULL)
       va_start (arg, pszFmt);
    return vFormatString ((LONG)psz, pszFmt, arg);
 }
@@ -220,7 +222,7 @@ LPTSTR cdecl FormatString (LPCTSTR psz, LPCTSTR pszFmt, ...)
 LPTSTR cdecl FormatString (int ids, LPCTSTR pszFmt, ...)
 {
    va_list  arg;
-   if (pszFmt != NULL)
+   //if (pszFmt != NULL)
       va_start (arg, pszFmt);
    return vFormatString ((LONG)ids, pszFmt, arg);
 }
@@ -371,7 +373,8 @@ LPTSTR cdecl vFormatString (LONG pszSource, LPCTSTR pszFmt, va_list arg)
       if ((psz = (LPTSTR)lstrchr (&szFmt[1], TEXT('%'))) != NULL)
          *psz = 0;
 
-      for (int ii = 1; szFmt[ii] == TEXT('-') || szFmt[ii] == TEXT(','); ii++)
+      int ii;
+      for (ii = 1; szFmt[ii] == TEXT('-') || szFmt[ii] == TEXT(','); ii++)
          ;
       cchMin = _ttol (&szFmt[ii]);
 
@@ -820,21 +823,21 @@ BOOL FormatTime (LPTSTR pszTarget, LPTSTR pszFormatUser, SYSTEMTIME *pst, BOOL f
       }
    else
       {
-      GetTimeFormat (LOCALE_USER_DEFAULT, 0,              &lt, NULL, szTime, cchRESOURCE);
-      GetDateFormat (LOCALE_USER_DEFAULT, DATE_SHORTDATE, &lt, NULL, szDate, cchRESOURCE);
+      GetTimeFormat (LOCALE_USER_DEFAULT, 0, &lt, "HH:mm:ss", szTime, cchRESOURCE);
+      GetDateFormat (LOCALE_USER_DEFAULT, 0, &lt, "yyyy-MM-dd", szDate, cchRESOURCE);
 
       if (fShowTime && fShowDate)
          {
-         lstrcat (szTime, TEXT(" "));
-         lstrcat (szTime, szDate);
+         lstrcat (szDate, TEXT(" "));
+         lstrcat (szDate, szTime);
          }
-      else if (fShowDate && !fShowTime)
+      else if (!fShowDate && fShowTime)
          {
-         lstrcpy (szTime, szDate);
+         lstrcpy (szDate, szTime);
          }
       }
 
-   wsprintf (pszTarget, FixFormatString (pszFormatUser), szTime);
+   wsprintf (pszTarget, FixFormatString (pszFormatUser), szDate);
    return rc;
 }
 
@@ -962,7 +965,8 @@ void FormatBytes (LPTSTR pszTarget, LPTSTR pszFormatUser, double cb)
       {
       lstrcpy (szFormat, pszFormatUser);
 
-      for (LPTSTR pch = &szFormat[1]; *pch; ++pch)
+      LPTSTR pch;
+      for (pch = &szFormat[1]; *pch; ++pch)
          {
          if (!isfmtgarbage(*pch))
             {
@@ -988,8 +992,8 @@ void FormatBytes (LPTSTR pszTarget, LPTSTR pszFormatUser, double cb)
             // Cheesy bit: if all we have are 0's and "."'s, just
             // make the number read "0".
             //
-
-   for (TCHAR *pch = szJustTheNumber; *pch; ++pch)
+   TCHAR * pch;
+   for (pch = szJustTheNumber; *pch; ++pch)
       {
       if (*pch != TEXT('0') && *pch != TEXT('.'))
          break;
@@ -1654,7 +1658,8 @@ LPTSTR FixFormatString (LPTSTR pszFormat)
       {
       lstrcpy (szFormat, pszFormat);
 
-      for (LPTSTR pch = &szFormat[1]; *pch; ++pch)
+      LPTSTR pch;
+      for (pch = &szFormat[1]; *pch; ++pch)
          {
          if (!isfmtgarbage(*pch))
             {
